#region Copyright Syncfusion Inc. 2001 - 2006
//
//  Copyright Syncfusion Inc. 2001 - 2006. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.OleDb;
using System.IO;
using System.Text;
using System.Diagnostics;

using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;

namespace GGC_HideTable
{
	public class GroupingEngineFactory : GridEngineFactoryBase
	{
		public override GridEngine CreateEngine(){return new GroupingEngine();}
	}

	public class GroupingEngine : GridEngine
	{
		public override ChildTable CreateChildTable(Element parent)
		{return new GroupingChildTable(parent);}

		public override Group CreateGroup(Section parent)
		{return new GroupingGroup(parent);}
	}

	
	#region ForTable.....................
	public class GroupingChildTable : ChildTable, IGridGroupOptionsSource
	{
		public GroupingChildTable(Element parent): base(parent){}

		public delegate void TableSummaryChanged(object sender,Syncfusion.Collections.BinaryTree.ITreeTableSummary[] ITableSummary);
		public event TableSummaryChanged SummaryChanged;
		
		public override Syncfusion.Collections.BinaryTree.ITreeTableSummary[] GetSummaries(Table parentTable, out bool summaryChanged)
		{
			Syncfusion.Collections.BinaryTree.ITreeTableSummary[] ITableSummary = base.GetSummaries (parentTable, out summaryChanged);
			if( summaryChanged)
				if( SummaryChanged != null)
					SummaryChanged(this,ITableSummary);
			return ITableSummary;
		}

		#region IGridGroupOptionsSource Members

		GridGroupOptionsStyleInfo groupOptions;

		public void RaiseGroupOptionsChanging(GridGroupOptionsChangedEventArgs e)
		{
		}

		public void RaiseGroupOptionsChanged(GridGroupOptionsChangedEventArgs e)
		{
			this.InvalidateCounterTopDown(true);
		}

		public bool HasGroupOptions
		{
			get
			{
				return groupOptions != null;
			}
		}

		/// <override/>
		public new GridEngine Engine
		{
			get
			{
				return (GridEngine) base.Engine;
			}
		}

		/// <summary>
		/// Lets you control the look of inner groups like whether the Caption Row is visible, or what CaptionText is.
		/// </summary>
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		[Browsable(true)]
		public GridGroupOptionsStyleInfo GroupOptions
		{
			get
			{
				if (groupOptions == null)
				{
					GridGroupOptionsType t = GridGroupOptionsType.Groups;
					if (this.IsTopLevelGroup)
						t = GridGroupOptionsType.TopLevelGroup;
					else //if (this is ChildTable)
						t = GridGroupOptionsType.ChildTable;

					groupOptions = new GridGroupOptionsStyleInfo(new GridGroupOptionsStyleInfoIdentity((IGridGroupOptionsSource) this, t));
				}
				return groupOptions;
			}
			set
			{
				GroupOptions.CopyFrom(value);
			}
		}

		public IGridGroupOptionsSource GetParentGroupOptionsSource()
		{
			if (this.CategoryColumns.Count > 0)
			{
				GridColumnDescriptor column = ((GridTableDescriptor) this.ParentTableDescriptor).Columns[this.CategoryColumns[0].Name];
				if (column != null)
					return column;
			}
			return (IGridGroupOptionsSource) this.ParentTableDescriptor;
		}

		#endregion

		#region OnIntializeSection
		protected override void OnInitializeSections(bool hasRecords, SortColumnDescriptorCollection fields)
		{
			// Caption
			this.Sections.Add(this.ParentTableDescriptor.CreateCaptionSection(this));

			// Column Headers
			this.Sections.Add(this.ParentTableDescriptor.CreateColumnHeaderSection(this));

			
			// AddNewRecord
			AddNewRecordSection addNewRecordSectionBeforeDetails = this.ParentTableDescriptor.CreateAddNewRecordSection(this);
			addNewRecordSectionBeforeDetails.IsBeforeDetails = true;
			this.Sections.Add(addNewRecordSectionBeforeDetails);

			// Details (Records or Groups)
			if (hasRecords)
				this.Sections.Add(this.ParentTableDescriptor.CreateRecordsDetails(this, fields));
			else
				this.Sections.Add(this.ParentTableDescriptor.CreateGroupsDetails(this, fields));

			// Summary
			this.Sections.Add(this.ParentTableDescriptor.CreateSummarySection(this));


		}		
		#endregion

	}

	#endregion

	#region ForGroup.....................
	public class GroupingGroup : Group, IGridGroupOptionsSource
	{
		public GroupingGroup(Section parent): base(parent){}
		public delegate void TableSummaryChanged(object sender,Syncfusion.Collections.BinaryTree.ITreeTableSummary[] ITableSummary);
		public event TableSummaryChanged SummaryChanged;

		public override Syncfusion.Collections.BinaryTree.ITreeTableSummary[] GetSummaries(Table parentTable, out bool summaryChanged)
		{
			Syncfusion.Collections.BinaryTree.ITreeTableSummary[] IGroupTableSummary = base.GetSummaries (parentTable, out summaryChanged);
			if( summaryChanged )
			{
				if( SummaryChanged != null)
						SummaryChanged(this,IGroupTableSummary);
			}
			return IGroupTableSummary;
		}

		#region IGridGroupOptionsSource Members

		GridGroupOptionsStyleInfo groupOptions;

		public void RaiseGroupOptionsChanging(GridGroupOptionsChangedEventArgs e)
		{
		}

		public void RaiseGroupOptionsChanged(GridGroupOptionsChangedEventArgs e)
		{
			this.InvalidateCounterTopDown(true);
		}

		public bool HasGroupOptions
		{
			get
			{
				return groupOptions != null;
			}
		}

		/// <override/>
		public new GridEngine Engine
		{
			get
			{
				return (GridEngine) base.Engine;
			}
		}

		/// <summary>
		/// Lets you control the look of inner groups like whether the Caption Row is visible, or what CaptionText is.
		/// </summary>
		[DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Content)]
		[Browsable(true)]
		public GridGroupOptionsStyleInfo GroupOptions
		{
			get
			{
				if (groupOptions == null)
				{
					GridGroupOptionsType t = GridGroupOptionsType.Groups;
					if (this.IsTopLevelGroup)
						t = GridGroupOptionsType.TopLevelGroup;
					else //if (this is ChildTable)
						t = GridGroupOptionsType.ChildTable;

					groupOptions = new GridGroupOptionsStyleInfo(new GridGroupOptionsStyleInfoIdentity((IGridGroupOptionsSource) this, t));
				}
				return groupOptions;
			}
			set
			{
				GroupOptions.CopyFrom(value);
			}
		}

		public IGridGroupOptionsSource GetParentGroupOptionsSource()
		{
			if (this.CategoryColumns.Count > 0)
			{
				GridColumnDescriptor column = ((GridTableDescriptor) this.ParentTableDescriptor).Columns[this.CategoryColumns[0].Name];
				if (column != null)
					return column;
			}
			return (IGridGroupOptionsSource) this.ParentTableDescriptor;
		}

		#endregion

		#region OnIntialize
		protected override void OnInitializeSections(bool hasRecords, SortColumnDescriptorCollection fields)
		{
			// Caption
			this.Sections.Add(this.ParentTableDescriptor.CreateCaptionSection(this));
			// AddNewRecord
			AddNewRecordSection addNewRecordSectionBeforeDetails = this.ParentTableDescriptor.CreateAddNewRecordSection(this);
			addNewRecordSectionBeforeDetails.IsBeforeDetails = true;
			this.Sections.Add(addNewRecordSectionBeforeDetails);

			// Details (Records or Groups)
			if (hasRecords)
				this.Sections.Add(this.ParentTableDescriptor.CreateRecordsDetails(this, fields));
			else
				this.Sections.Add(this.ParentTableDescriptor.CreateGroupsDetails(this, fields));
			// Summary
			this.Sections.Add(this.ParentTableDescriptor.CreateSummarySection(this));
		}		
		#endregion


		public override bool IsChildVisible(Element el)
		{
			if (el is CaptionSection &&  this.IsExpanded ) 
				return false;
			else
				return base.IsChildVisible(el);// Otherwise default behavior:
		}
	}
	#endregion
}

